﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace Ukazka2Defektu.Stetce
{
    /// <summary>
    /// Interakční logika pro EfektLinearGradientBrush.xaml
    /// </summary>
    public partial class EfektLinearGradientBrush : UserControl
    {
        private bool captured = false;
        private double x_canvasS, y_canvasS, x_canvasE, y_canvasE;
        private double xStart, yStart, xEnd, yEnd;
        private UIElement bodStart = null;
        private UIElement bodEnd = null;
        private Point startPoint;
        private Point endPoint;
        private Line cara = new Line();
        private double polomer = 0;
        private double minX, maxX, minY, maxY;
        private double minXram, maxXram, minYram, maxYram;

        public EfektLinearGradientBrush()
        {
            InitializeComponent();

            polomer = start.Width / 2;

            xStart = (rctZaklad.Width / 2);
            yStart = 0;
            xEnd = (rctZaklad.Width / 2);
            yEnd = rctZaklad.Height;

            minX = 0;
            maxX = rctZaklad.Width;
            minY = 0;
            maxY = rctZaklad.Height;

            minXram = -polomer;
            maxXram = rctZaklad.Width + polomer;
            minYram = -polomer;
            maxYram = rctZaklad.Height + polomer;

            cara = new Line()
            {
                X1 = xStart,
                Y1 = yStart,
                X2 = xEnd,
                Y2 = yEnd,
                StrokeThickness = 1,
                Stroke = new SolidColorBrush(Colors.Black)
            };
            cnsCan.Children.Add(cara);

            Canvas.SetLeft(start, xStart - polomer);
            Canvas.SetTop(start, yStart - polomer);
            Canvas.SetLeft(end, xEnd - polomer);
            Canvas.SetTop(end, yEnd - polomer);

            NaplnData();

            cbxSpread.Items.Add("Pad");
            cbxSpread.Items.Add("Reflect");
            cbxSpread.Items.Add("Repeat");
            cbxSpread.SelectedValue = "Pad";
        }

        /// <summary>
        /// Načtení počátečních souřadnic bodu Start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NactiSourStart(object sender, MouseButtonEventArgs e)
        {
            bodStart = (UIElement)sender;
            Mouse.Capture(bodStart);
            captured = true;
            xStart = Canvas.GetLeft(bodStart);
            x_canvasS = e.GetPosition(cnsCan).X;
            yStart = Canvas.GetTop(bodStart);
            y_canvasS = e.GetPosition(cnsCan).Y;
        }

        /// <summary>
        /// Posun bodu Start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PosunStart(object sender, MouseEventArgs e)
        {
            if (captured)
            {
                double x = e.GetPosition(cnsCan).X;
                double y = e.GetPosition(cnsCan).Y;

                if (x >= minXram && x <= maxXram)
                {
                    xStart += x - x_canvasS;
                    Canvas.SetLeft(bodStart, xStart);
                    x_canvasS = x;
                }

                if (y >= minYram && y <= maxYram)
                {
                    yStart += y - y_canvasS;
                    Canvas.SetTop(bodStart, yStart);
                    y_canvasS = y;
                }

                cnsCan.Children.Remove(cara);
                cara = new Line()
                {
                    X1 = xStart + polomer,
                    Y1 = yStart + polomer,
                    X2 = Canvas.GetLeft(end) + polomer,
                    Y2 = Canvas.GetTop(end) + polomer,
                    StrokeThickness = 1,
                    Stroke = new SolidColorBrush(Colors.Black)
                };

                cnsCan.Children.Add(cara);

                NaplnData();
            }
        }

        /// <summary>
        /// Nové umístění bodu Start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ZastavStart(object sender, MouseButtonEventArgs e)
        {
            Mouse.Capture(null);
            captured = false;
        }

        /// <summary>
        /// Načtení počátečních souřadnic bodu End
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NactiSourEnd(object sender, MouseButtonEventArgs e)
        {
            bodEnd = (UIElement)sender;
            Mouse.Capture(bodEnd);
            captured = true;
            xEnd = Canvas.GetLeft(bodEnd);
            x_canvasE = e.GetPosition(cnsCan).X;
            yEnd = Canvas.GetTop(bodEnd);
            y_canvasE = e.GetPosition(cnsCan).Y;
        }

        /// <summary>
        /// Posun bodu End
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PosunEnd(object sender, MouseEventArgs e)
        {
            if (captured)
            {
                double x = e.GetPosition(cnsCan).X;
                double y = e.GetPosition(cnsCan).Y;

                if (x >= minXram && x <= maxXram)
                {
                    xEnd += x - x_canvasE;
                    Canvas.SetLeft(bodEnd, xEnd);
                    x_canvasE = x;
                }

                if (y >= minYram && y <= maxYram)
                {
                    yEnd += y - y_canvasE;
                    Canvas.SetTop(bodEnd, yEnd);
                    y_canvasE = y;
                }

                cnsCan.Children.Remove(cara);
                cara = new Line()
                {
                    X1 = Canvas.GetLeft(start) + polomer,
                    Y1 = Canvas.GetTop(start) + polomer,
                    X2 = xEnd + polomer,
                    Y2 = yEnd + polomer,
                    StrokeThickness = 1,
                    Stroke = new SolidColorBrush(Colors.Black)
                };

                cnsCan.Children.Add(cara);
                NaplnData();
            }
        }

        /// <summary>
        /// Nové umístění bodu End
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ZastavEnd(object sender, MouseButtonEventArgs e)
        {
            Mouse.Capture(null);
            captured = false;
        }

        /// <summary>
        /// Nastavení vlastnosti SpreadMethod
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ZmenaSpread(object sender, SelectionChangedEventArgs e)
        {
            if (cbxSpread != null)
            {
                string vybrano = cbxSpread.SelectedItem.ToString();
                switch (vybrano)
                {
                    case "Pad":
                        linearni.SpreadMethod = GradientSpreadMethod.Pad;
                        break;
                    case "Reflect":
                        linearni.SpreadMethod = GradientSpreadMethod.Reflect;
                        break;
                    case "Repeat":
                        linearni.SpreadMethod = GradientSpreadMethod.Repeat;
                        break;
                    default:
                        break;
                }
            }
        }


        /// <summary>
        /// Nastavení StartPointu a EndPointu
        /// </summary>
        private void NaplnData()
        {
            double xS = Math.Round(xStart / maxX, 2);
            if (xStart < 0)
            {
                xS = 0;
            }
            if (xStart > rctZaklad.Width)
            {
                xS = 1;
            }
            double yS = Math.Round(yStart / maxY, 2);
            if (yStart < 0)
            {
                yS = 0;
            }
            if (yStart > rctZaklad.Height)
            {
                yS = 1;
            }
            double xE = Math.Round(xEnd / maxX, 2);
            if (xEnd < 0)
            {
                xE = 0;
            }
            if (xEnd > rctZaklad.Width)
            {
                xE = 1;
            }
            double yE = Math.Round(yEnd / maxY, 2);
            if (yEnd < 0)
            {
                yE = 0;
            }
            if (yEnd > rctZaklad.Height)
            {
                yE = 1;
            }

            tbxXstart.Text = xS.ToString();
            tbxYstart.Text = yS.ToString();
            tbxXend.Text = xE.ToString();
            tbxYend.Text = yE.ToString();

            startPoint = new Point(xS, yS);
            endPoint = new Point(xE, yE);

            linearni.StartPoint = startPoint;
            linearni.EndPoint = endPoint;
        }
    }
}
